#include "xmame.h"
#include "devices.h"
#include "gp2x_joy.h"
#include "gp2x_ui.h"
#include "gp2x_opt.h"
#include "gp2x_vkeyb.h"
#include "unix/keycodes.h"

///////////////////////////////////

int gp2x_buttons[ 11 ] = { 
    GP2X_BUTTON_A,              /* called button 1 within xmess */
    GP2X_BUTTON_Y,              /* called button 2 within xmess, etc */
    GP2X_BUTTON_X,
    GP2X_BUTTON_B,
    GP2X_BUTTON_L,
    GP2X_BUTTON_R,
    GP2X_BUTTON_START,
    GP2X_BUTTON_SELECT,
    GP2X_BUTTON_VOLUP,
    GP2X_BUTTON_VOLDOWN,
    GP2X_BUTTON_CLICK,
};

struct rc_option joy_SDL_opts[] = {
   { NULL,		NULL,		rc_end,	NULL,
     NULL,		0,			0,		NULL,
     NULL 
   }
};

#if defined SDL_JOYSTICK

#include <SDL.h>

#define POLL_COUNT_PUMP 1   /* TODO: I forget what this was for */

void joy_SDL_init( void );
void joy_SDL_poll( void );

SDL_Joystick* joystick[JOY_MAX];

void 
joy_SDL_init( void )
{
	int j;

#ifdef __GP2X_HOST__

        /* Dummy data just to make host OK when no joystick plugged in */

        joy_data[0].fd          = 1;
        joy_data[0].num_buttons = 11;
        joy_data[0].num_axes    = 2;

        joy_poll_func = joy_SDL_poll;

        return;
#endif

	if ( SDL_Init( SDL_INIT_JOYSTICK ) < 0 )
		printf ( "SDL: JoyStick init Error!! " );
	else 
		printf( "SDL: joystick interface initialization...\n" );

    printf( "SDL: The name of the joystick : %s\n", SDL_JoystickName( 0 ) );

    /* Only one joystick on the gp2x */

    joystick[0] = SDL_JoystickOpen( 0 );

    if ( !joystick[0] )
        printf( "SDL:  SDL_JoystickOpen() FAILED!!\n" );

    if ( joystick[0] )
    {
        SDL_JoystickEventState( SDL_ENABLE );

        /* Set the file descriptor to a dummy value. */

        joy_data[0].fd          = 1;
        joy_data[0].num_buttons = 11;
        joy_data[0].num_axes    = 2;

        printf( "SDL:  the joystick has %d axes and %d buttons\n", joy_data[0].num_axes, joy_data[0].num_buttons );

        if ( joy_data[0].num_buttons > JOY_BUTTONS )
            joy_data[0].num_buttons = JOY_BUTTONS;

        for ( j = 0; j < joy_data[0].num_axes; j++ )
        {
            joy_data[0].axis[j].min = -32768;
            joy_data[0].axis[j].max =  32768;
            joy_data[0].axis[j].mid =  0;
        }
    }
    else
        printf( "SDL: the joystick init FAIL!!\n" );

	joy_poll_func = joy_SDL_poll;
}

///////////////////////////////////

void 
joy_SDL_poll( void )
{
#ifdef SDL_JOYSTICK

	int j;

#ifdef __GP2X_HOST__

    if ( gp2x_cur_opts.vkeyb && gp2x_display_vkeyb )
    {
        /* Hook all joystick input when keyboard is being shown */

        if ( osd_get_code_value( KEY_LEFT ) )      /* left */
            gp2x_vkeyb_handle_input( GP2X_BUTTON_LEFT, gp2x_cur_opts.vkeyb );

        else if ( osd_get_code_value( KEY_RIGHT ) )      /* right */
            gp2x_vkeyb_handle_input( GP2X_BUTTON_RIGHT, gp2x_cur_opts.vkeyb );

        else if ( osd_get_code_value( KEY_DOWN ) )      /* down */
            gp2x_vkeyb_handle_input( GP2X_BUTTON_DOWN, gp2x_cur_opts.vkeyb );

        else if ( osd_get_code_value( KEY_UP ) )      /* up */
            gp2x_vkeyb_handle_input( GP2X_BUTTON_UP, gp2x_cur_opts.vkeyb );

        else if ( osd_get_code_value( KEY_ENTER ) )      /* A button */
            gp2x_vkeyb_handle_input( GP2X_BUTTON_A, gp2x_cur_opts.vkeyb );

        else if ( osd_get_code_value( KEY_RSHIFT ) )      /* B button */
            gp2x_vkeyb_handle_input( GP2X_BUTTON_B, gp2x_cur_opts.vkeyb );

        else if ( osd_get_code_value( KEY_BACKSPACE ) )      /* joystick button */
            gp2x_vkeyb_handle_input( GP2X_BUTTON_CLICK, gp2x_cur_opts.vkeyb );

        else
            gp2x_vkeyb_pump_event();
    }

#else

	SDL_JoystickUpdate();

    if ( joy_data[0].fd )
    {
        if ( gp2x_cur_opts.vkeyb && gp2x_display_vkeyb )
        {
            /* Hook all joystick input when keyboard is being shown */

            if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_LEFT ) )
                gp2x_vkeyb_handle_input( GP2X_BUTTON_LEFT, gp2x_cur_opts.vkeyb );

            else if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_RIGHT ) )
                gp2x_vkeyb_handle_input( GP2X_BUTTON_RIGHT, gp2x_cur_opts.vkeyb );

            else if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_DOWN ) )
                gp2x_vkeyb_handle_input( GP2X_BUTTON_DOWN, gp2x_cur_opts.vkeyb );

            else if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_UP ) )
                gp2x_vkeyb_handle_input( GP2X_BUTTON_UP, gp2x_cur_opts.vkeyb );

            else if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_A ) )      /* select */
                gp2x_vkeyb_handle_input( GP2X_BUTTON_A, gp2x_cur_opts.vkeyb );

            else if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_B ) )      /* select w/ shift */
                gp2x_vkeyb_handle_input( GP2X_BUTTON_B, gp2x_cur_opts.vkeyb );

            else if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_CLICK ) )  /* toggle caps lock */
                gp2x_vkeyb_handle_input( GP2X_BUTTON_CLICK, gp2x_cur_opts.vkeyb );

            else
                gp2x_vkeyb_pump_event();

            /* joystick input that is *NOT* used by the virtual keyboard for input can be updated.
             * This means X, Y, Start, Select, shoulder buttons, and volume buttons.
             */

            for ( j = 0; j < joy_data[0].num_buttons; j++ )
            {
                if ( gp2x_buttons[ j ] != GP2X_BUTTON_A && gp2x_buttons[ j ] != GP2X_BUTTON_B && gp2x_buttons[ j ] != GP2X_BUTTON_CLICK )
                {
                    joy_data[0].buttons[j] = SDL_JoystickGetButton( joystick[0], gp2x_buttons[ j ] );
                }
            }
        }

        else
        {
            for ( j = 0; j < joy_data[0].num_buttons; j++ )
                joy_data[0].buttons[j] = SDL_JoystickGetButton( joystick[0], gp2x_buttons[ j ] );

            /* This should allow analog joy setups to at least partially work */

            joy_data[0].axis[0].val = 0;
            joy_data[0].axis[1].val = 0;

            if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_LEFT ) )
                joy_data[0].axis[0].val = -32768;

            if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_RIGHT ) )
                joy_data[0].axis[0].val = 32768;

            if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_UP ) )
                joy_data[0].axis[1].val = -32768;

            if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_DOWN ) )
                joy_data[0].axis[1].val = 32768;

            if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_UPLEFT ) )
            {
                joy_data[0].axis[1].val = -16834;
                joy_data[0].axis[0].val = -16834;
            }

            if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_UPRIGHT ) )
            {
                joy_data[0].axis[1].val = -16834;
                joy_data[0].axis[0].val =  16834;
            }

            if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_DOWNLEFT ) )
            {
                joy_data[0].axis[1].val =  16834;
                joy_data[0].axis[0].val = -16834;
            }

            if ( SDL_JoystickGetButton( joystick[0], GP2X_BUTTON_DOWNRIGHT ) )
            {
                joy_data[0].axis[1].val = 16834;
                joy_data[0].axis[0].val = 16834;
            }
        }
    }
#endif

    /* 
     * This is as good a place as any since it's polled and it's not
     * in xmess's src tree proper.  Must be @ end of joystick update.
     */

    if ( input_ui_pressed( GP2X_VKEYBD_MAPPING ) )
    {
        if ( gp2x_cur_opts.vkeyb )
        {
            if ( gp2x_display_vkeyb )
            {
                gp2x_clean_up_keyboard = 2;
                gp2x_vkeyb_cleanup();
            }

            else
            {
                gp2x_vkeyb_init( gp2x_cur_opts.vkeyb );
            }

            gp2x_display_vkeyb = !gp2x_display_vkeyb;
        }
    }

    if ( input_ui_pressed( GP2X_VOL_UP_MAPPING ) )
        gp2x_volume_change( 1 );

    if ( input_ui_pressed( GP2X_VOL_DOWN_MAPPING ) )
        gp2x_volume_change( -1 );

#endif
}

#endif
